import { aws_rds as rds, aws_secretsmanager as secretsmanager } from "aws-cdk-lib";
import { Construct } from "constructs";
import { CustomLambdaFunctionProps } from "../utils";
/**
 * An RDS instance with pgSTAC installed. This is a wrapper around the
 * `rds.DatabaseInstance` higher-level construct making use
 * of the BootstrapPgStac construct.
 */
export declare class PgStacDatabase extends Construct {
    db: rds.DatabaseInstance;
    pgstacSecret: secretsmanager.ISecret;
    constructor(scope: Construct, id: string, props: PgStacDatabaseProps);
    getParameters(instanceType: string, parameters: PgStacDatabaseProps["parameters"]): DatabaseParameters;
}
export interface PgStacDatabaseProps extends rds.DatabaseInstanceProps {
    /**
     * Name of database that is to be created and onto which pgSTAC will be installed.
     *
     * @default pgstac
     */
    readonly pgstacDbName?: string;
    /**
   * Prefix to assign to the generated `secrets_manager.Secret`
   *
   * @default pgstac
   */
    readonly secretsPrefix?: string;
    /**
     * Name of user that will be generated for connecting to the pgSTAC database.
     *
     * @default pgstac_user
     */
    readonly pgstacUsername?: string;
    /**
     * Lambda function Custom Resource properties. A custom resource property is going to be created
     * to trigger the boostrapping lambda function. This parameter allows the user to specify additional properties
     * on top of the defaults ones.
     *
     */
    readonly customResourceProperties?: {
        [key: string]: any;
    };
    /**
     * Optional settings for the bootstrapper lambda function. Can be anything that can be configured on the lambda function, but some will be overwritten by values defined here.
     *
     * @default - defined in the construct.
     */
    readonly bootstrapperLambdaFunctionOptions?: CustomLambdaFunctionProps;
}
export interface DatabaseParameters {
    /**
     * @default - LEAST({DBInstanceClassMemory/9531392}, 5000)
     */
    readonly maxConnections: string;
    /**
     * Note: This value is measured in 8KB blocks.
     *
     * @default '{DBInstanceClassMemory/32768}' 25% of instance memory, ie `{(DBInstanceClassMemory/(1024*8)) * 0.25}`
     */
    readonly sharedBuffers: string;
    /**
     * @default - 75% of instance memory
     */
    readonly effectiveCacheSize: string;
    /**
     * @default - shared buffers divided by max connections
     */
    readonly workMem: string;
    /**
     * @default - 25% of shared buffers
     */
    readonly maintenanceWorkMem: string;
    /**
     * @default 1024
     */
    readonly maxLocksPerTransaction: string;
    /**
     * @default 131172 (128 * 1024)
     */
    readonly tempBuffers: string;
    /**
     * @default 1
     */
    readonly seqPageCost: string;
    /**
     * @default 1.1
     */
    readonly randomPageCost: string;
}
